import json
import re

from pymongo import MongoClient
from bson.objectid import ObjectId

from trend_etl.functions.index import s3_list_objects

secrets_file = ".mongo_uri.json"
database = "redditpl"
coll = "reddit_archive_paths"


def main() -> None:
    with open(secrets_file, "r") as file:
        data = file.read()

    secret = json.loads(data)

    paths_added = 0

    with MongoClient(secret["uri"]) as client:  # type: ignore
        db = client.get_database(database)
        for page in s3_list_objects("trendshare-reddit"):
            for obj in page["Contents"]:
                find_result = re.findall(r"job_id=([^/]+)", obj["Key"])
                assert len(find_result) == 1
                if not ObjectId.is_valid(find_result[0]):
                    print(f"path contains invalid mongo id: {find_result[0]}")
                    continue

                paths_added += 1

                if "kind=articles" in obj["Key"]:
                    db.get_collection(coll).update_one(
                        {"_id": obj["Key"]},
                        {"$setOnInsert": {"migrated": False, "tag": "articles"}},
                        upsert=True,
                    )
                elif "kind=comments" in obj["Key"]:
                    db.get_collection(coll).update_one(
                        {"_id": obj["Key"]},
                        {"$setOnInsert": {"migrated": False, "tag": "comments"}},
                        upsert=True,
                    )
                elif "kind=subredditmetadatasnapshot" in obj["Key"]:
                    db.get_collection(coll).update_one(
                        {"_id": obj["Key"]},
                        {
                            "$setOnInsert": {
                                "migrated": False,
                                "tag": "subredditmetadatasnapshot",
                            }
                        },
                        upsert=True,
                    )
                else:
                    raise ValueError(f"unexpected kind for key: {obj['Key']}")
                if paths_added % 100_000 == 0:
                    print(f"paths added so far: {paths_added}")

    print(f"ok. total paths added: {paths_added}")


if __name__ == "__main__":
    main()
