from pyspark.sql import DataFrame

# from pyspark.sql.column import Column
from pyspark.sql.functions import (
    col,
    get_json_object,
    expr,
)
from pyspark.sql.types import StringType

import trend_etl.schema.fields as fields
import trend_etl.schema.args as arguments


# arguments:
#   input_df: the dataframe from a bronze table
#   id_json_path: json path to the source (i.e. facebook, reddit) id for the document
#   target_table: the specific idents table `input_df` is
# return: a dataframe in the schema expected for a *_idents mapping table
def from_bronze_get_idents_df(
    input_df: DataFrame, id_json_path: str, target_table: str
) -> DataFrame:
    if target_table in (
        arguments.tables[arguments.tables_group_idents],
        arguments.tables[arguments.tables_subreddit_idents],
        arguments.tables[arguments.tables_guild_idents],
    ):
        uuid_alias = fields.idents_group_id_field
    else:
        uuid_alias = fields.idents_doc_id_field
    id_col = get_json_object(
        col(fields.bronze_value_field).cast(StringType()), id_json_path
    )
    return input_df.select(
        expr("uuid()").alias(uuid_alias),
        id_col.alias(fields.idents_source_id_field),
    )
