from pyspark.sql import DataFrame

from pyspark.sql.functions import (
    col,
    get_json_object,
    lit,
)
from pyspark.sql.types import StringType, DateType

import trend_etl.schema.fields as fields
import trend_etl.schema.args as arguments


def from_bronze_get_daily_ingestion_count(
    streaming_bronze: DataFrame,
    group_idents: DataFrame,
    group_source_id_json_path: str,
    statement_type_index: int,
) -> DataFrame:
    id_col = get_json_object(
        col(fields.bronze_value_field).cast(StringType()), group_source_id_json_path
    )
    parsed_df = streaming_bronze.select(
        id_col.alias(fields.idents_source_id_field),
        col(fields.bronze_timestamp_field)
        .cast(DateType())
        .alias(fields.collected_day_field),
    )
    joined_df = parsed_df.join(
        group_idents,
        fields.idents_source_id_field,
        "inner",  # filter out those statements without a groups's source id
    )

    agg_df = (
        joined_df.groupBy(fields.idents_group_id_field, fields.collected_day_field)
        .count()
        .withColumn(
            fields.statement_type_field,
            lit(arguments.statement_types[statement_type_index]),
        )
    )

    return agg_df


def from_bronze_get_job_run_ingestion_count(
    streaming_bronze: DataFrame,
    group_idents: DataFrame,
    group_source_id_json_path: str,
    statement_type_index: int,
    job_run_id: str,
) -> DataFrame:
    id_col = get_json_object(
        col(fields.bronze_value_field).cast(StringType()), group_source_id_json_path
    )
    parsed_df = streaming_bronze.select(
        id_col.alias(fields.idents_source_id_field),
    )
    joined_df = parsed_df.join(
        group_idents,
        fields.idents_source_id_field,
        "inner",  # filter out those statements without a groups's source id
    )

    agg_df = (
        joined_df.groupBy(fields.idents_group_id_field)
        .count()
        .withColumns(
            {
                fields.statement_type_field: lit(
                    arguments.statement_types[statement_type_index]
                ),
                fields.job_run_id_field: lit(job_run_id),
            }
        )
    )

    return agg_df
