from pyspark.sql import DataFrame

from pyspark.sql.functions import col, lit, struct
from pyspark.sql.types import StringType

import trend_etl.schema.fields as fields
from trend_etl.functions.index import get_collected_day, parse_reactions


#
# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
# return: a dataframe in the schema expected for bronze tables
def from_kafka_get_bronze_reactions_df(
    input_df: DataFrame, target_topic: str
) -> DataFrame:
    return input_df.select(
        col(fields.kafka_value_field).alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_collected_day(col(fields.kafka_timestamp_field)).alias(
            fields.collected_day_field
        ),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   message_idents: message uuid idents table
def from_bronze_get_reactions_df(
    streaming_bronze: DataFrame,
    message_idents: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_reactions(col(fields.bronze_value_field).cast(StringType())).alias(
            "value"
        ),
        col(fields.bronze_timestamp_field),
    )

    flattened_df = parsed_df.select(
        col("value.*"),
        col(fields.bronze_timestamp_field),
    )

    result_df = (
        flattened_df.withColumn(
            fields.source_keys_field, struct(col("message_id").alias("message_id"))
        )
        .join(message_idents.withColumnRenamed("id", "message_id"), "message_id")
        .drop("message_id")
    )

    return result_df
